//
// Logging Interface for LCDHype
//
// $Id: LogInterface.pas 6 2011-05-14 13:38:55Z LostSoul $
//

unit LogInterface;

{$Z4}
{$A4} // we use 4 byte alignment because packed record costs too much performance

interface

const
  coLogInterfaceRevision = '$Revision: 6 $';

type
  // structure

  TLogInterfaceEntryKind = (ekUnknown = 0,
                            ekInfo = 1,
                            ekAttention = 2,
                            ekError = 3);

  PLogInterfaceEntry = ^TLogInterfaceEntry;
  TLogInterfaceEntry = record
    StructureSize: Longword;
    Kind: TLogInterfaceEntryKind;
    Source: PWideChar;
    Text: PWideChar;
  end;

  // callbacks

  TLogInterfaceAddEntryCallBack = function (ALogEntry: PLogInterfaceEntry): Boolean; stdcall;

function KindToStr(const AKind: TLogInterfaceEntryKind): String;
function StrToKind(const AValue: String): TLogInterfaceEntryKind;

implementation

uses
  SysUtils;

function KindToStr(const AKind: TLogInterfaceEntryKind): String;
begin
  case AKind of
    ekInfo: result := 'Info';
    ekAttention: result := 'Attention';
    ekError: result := 'Error';
    else
      result := 'Unknown';
  end;
end;

function StrToKind(const AValue: String): TLogInterfaceEntryKind;
var
  lValue: String;
begin
  lValue := Trim(AValue);

  if CompareText(lValue, 'Info') = 0 then
    result := ekInfo
  else
  if CompareText(lValue, 'Attention') = 0 then
    result := ekAttention
  else
  if CompareText(lValue, 'Error') = 0 then
    result := ekError
  else
    result := ekUnknown;
end;

end.
