//
// LCDHype output device / controller interface, requires LCDHype version 0.7.2 or above
//
// $Id: ControllerInterface.pas 21 2013-02-06 20:22:19Z LostSoul $
//

unit ControllerInterface;

{$Z4}
{$A4} // we use 4 byte alignment because packed record costs too much performance

interface

const
  coControllerInterfaceRevision = '$Revision: 21 $';

type
  // structures

  PControllerInterfaceCustomChar = ^TControllerInterfaceCustomChar;
  TControllerInterfaceCustomChar = record
    StructureSize: LongWord;
    Position: LongWord;       // states the position where the custom char should be stored in LCD memory
    Width: LongWord;          // states the width of the custom char
    Height: LongWord;         // states the height of the custom char
    Pixel: PByte;             // holds the width * height custom char pixel bytes, each byte is a pixel, 0 = off, 1 = on
  end;

  PControllerInterfaceSymbol = ^TControllerInterfaceSymbol;
  TControllerInterfaceSymbol = record
    StructureSize: LongWord;
    Index: LongWord;                      // symbol index
    State: Byte;                          // state of a symbol, currently only 0 for off, 1 for on is supported
  end;
                                        // Kind field of TControllerInterfaceInOutProperty   // Value field of TControllerInterface%Property, Description
  TControllerInterfaceGetPropertyKind = (gpName = 0,                                         // in: nil, return: PWideChar, states the driver name, dont include version when library supplies version info because lcdhype reads out version info automatically
                                         gpTextModeResolution = 1,                           // PControllerInterfaceResolution
                                         gpTextModeCharacterResolution = 2,                  // PControllerInterfaceResolution
                                         gpFontPitch = 3,                                    // PByte, fontpitch (in most cases GraphicModeWidth / TextModeWidth) of output device in pixels
                                         gpGraphicModeResolution = 4,                        // PControllerInterfaceResolution
                                         gpColorModeSupported = 5,                           // PBoolean
                                         gpConfigurationDialogSupported = 6,                 // PBoolean
                                         gpOutportCount = 7,                                 // PLongWord, number of outports supported
                                         gpSymbolCount = 8,                                  // PLongWord, number of supported symbols
                                         gpBacklightSliderIntervalCount = 9,                 // PLongWord, a numeric value >= 2 stating number of intervals used by backlight slider, if backlight slider is not supported then pass 0
                                         gpContrastSliderIntervalCount = 10,                 // PLongWord, a numeric value >= 2 stating number of intervals used by contrast slider, if contrast slider is not supported then pass 0
                                         gpReadyToReceive = 11,                              // PBoolean, property continuously polled, signals wether the output device is ready to receive new data or not
                                         gpCustomCharPosition = 12,                          // PControllerInterfaceCustomCharPosition, property requested by LCDHype to get the position of a custom char in the memory of the output device
                                         gpCustomCharCount = 13                              // PLongWord, number of supported custom chars
                                        );

  TControllerInterfaceSetPropertyKind = (spLibraryPath = 0,                                  // PWideChar, DLL path
                                         spConfigurationPath = 1,                            // PWideChar, path where configuration files should be stored
                                         spBacklightState = 2,                               // PBoolean, backlight on = true, backlight off = false
                                         spBacklightSliderValue = 3,                         // PLongWord
                                         spContrastState = 4,                                // PBoolean, contrast on = true, contrast off = false
                                         spContrastSliderValue = 5,                          // PLongWord
                                         spOutportState = 6,                                 // PControllerInterfaceOutportState
                                         spInvertedOutput = 7,                               // PBoolean
                                         spAddLogEntryCallback = 8,                          // TLogInterfaceAddEntryCallBack, call this method to send log to lcdhype
                                         spMainWindowHandle = 9,                             // PLongWord, window handle of application main window
                                         spOutputEnabled = 10,                               // PBoolean
                                         spApplicationVersion = 11                           // PControllerInterfaceApplicationVersion
                                        );

  PControllerInterfaceGetProperty = ^TControllerInterfaceGetProperty;
  TControllerInterfaceGetProperty = record
    StructureSize: LongWord;
    Kind: TControllerInterfaceGetPropertyKind;
    Value: Pointer;
  end;

  PControllerInterfaceSetProperty = ^TControllerInterfaceSetProperty;
  TControllerInterfaceSetProperty = record
    StructureSize: LongWord;
    Kind: TControllerInterfaceSetPropertyKind;
    Value: Pointer;
  end;

  PControllerInterfaceSetPropertyList = ^TControllerInterfaceSetPropertyList;
  TControllerInterfaceSetPropertyList = record
    StructureSize: LongWord;
    Length: Integer;
    Element: PControllerInterfaceSetProperty;
  end;

  PControllerInterfaceApplicationVersion = ^TControllerInterfaceApplicationVersion;
  TControllerInterfaceApplicationVersion = record
    StructureSize: LongWord;
    Major: LongWord;
    Minor: LongWord;
    Release: LongWord;
    Build: LongWord;
  end;

  PControllerInterfaceOutportState = ^TControllerInterfaceOutportState;
  TControllerInterfaceOutportState = record
    Index: LongWord;
    State: Boolean;
  end;

  PControllerInterfaceResolution = ^TControllerInterfaceResolution;
  TControllerInterfaceResolution = record
    Width: LongWord;
    Height: LongWord;
  end;

  PControllerInterfaceCustomCharPosition = ^TControllerInterfaceCustomCharPosition;
  TControllerInterfaceCustomCharPosition = record
    Internal: LongWord; // position internally used by LCDHype, first custom char begins at internal position 0
    Memory: LongWord;   // position where custom char is stored in memory of output device,
                        // when LCDHype passes 0 as internal position and the first custom char is stored at memory position 128 then the
                        // the field Memory should be filled with 128
  end;

  PControllerInterfaceTextCharacter = ^TControllerInterfaceTextCharacter;
  TControllerInterfaceTextCharacter = record
    StructureSize: LongWord;
    Character: WideChar;
    Underlined: Boolean;
  end;

  PControllerInterfaceText = ^TControllerInterfaceText;
  TControllerInterfaceText = record
    StructureSize: LongWord;
    Scan: PControllerInterfaceTextCharacter; // width * height text characters
    Width: LongWord;
    Height: LongWord;
    Column: LongWord; // states the column where to start output, currently LCDHype always passes 0
    Row: LongWord;    // states the row where to start output, currently LCDHype always passes 0
  end;

  PControllerInterfaceBitmapPixel = ^TControllerInterfaceBitmapPixel;
  TControllerInterfaceBitmapPixel = packed record
    Blue: Byte;
    Green: Byte;
    Red: Byte;
    Alpha: Byte; // alpha currently holds no valid value, so skip it
  end;

  PControllerInterfaceBitmap = ^TControllerInterfaceBitmap;
  TControllerInterfaceBitmap = record
    StructureSize: LongWord;
    Scan: PControllerInterfaceBitmapPixel;  // number of PBitmapPixel = width * height
    Width: LongWord;                        // number of pixels in horizontal direction
    Height: LongWord;                       // number of pixels in vertical direction
    X1: LongWord;                           // defines the left boundary of the rectangle within the bitmap that should be displayed
    Y1: LongWord;                           // defines the top boundary of the rectangle within the bitmap that should be displayed
    X2: LongWord;                           // defines the right boundary of the rectangle within the bitmap that should be displayed
    Y2: LongWord;                           // defines the bottom boundary of the rectangle within the bitmap that should be displayed
  end;

                                    // Kind field of TControllerInterfaceCommand   // Value field of TControllerInterfaceCommand, Description
  TControllerInterfaceCommandKind = (ckReserved = 0,                               // PByte, this kind of command is needed by very old drivers
                                     ckHandleBacklightOn = 1,                      // not used
                                     ckHandleBacklightOff = 2,                     // not used
                                     ckHandleContrastOn = 3,                       // not used
                                     ckHandleContrastOff = 4,                      // not used
                                     ckHandleOutportChanged = 5,                   // not used
                                     ckShowConfigurationDialog = 6,                // PControllerInterfaceConfigurationDialogParameter, when the user clicks
                                                                                   // on the "Advanced configuration" button in the controller configuration dialog
                                     ckHandleSuspending = 7,                       // not used, machine is going into suspended state
                                     ckHandleResuming = 8                          // not used, machine is resuming from supended state
                                    );

  PControllerInterfaceCommand = ^TControllerInterfaceCommand;
  TControllerInterfaceCommand = record
    StructureSize: LongWord;
    Kind: TControllerInterfaceCommandKind;
    Value: Pointer;
  end;

  PControllerInterfaceConfigurationDialogParameter = ^TControllerInterfaceConfigurationDialogParameter;
  TControllerInterfaceConfigurationDialogParameter = record
    StructureSize: LongWord;
    ParentWindow: THandle;
  end;

  PControllerInterfaceInitParameter = ^TControllerInterfaceInitParameter;
  TControllerInterfaceInitParameter = record
    StructureSize: LongWord;
    CallCount: LongWord;
  end;

  PControllerInterfaceCleanUpParameter = ^TControllerInterfaceCleanUpParameter;
  TControllerInterfaceCleanUpParameter = record
    StructureSize: LongWord;
  end;

  // method is called by LCDHype to get information from the driver, e.g. name or configuration,
  // please have a look at TControllerInterfaceGetPropertyKind to see which properties are supported
  TControllerInterfaceGetPropertyMethod = procedure (AProperty: PControllerInterfaceGetProperty); stdcall;

  // method is called by LCDHype to set serveral properties that may be needed by the driver to work correctly,
  // please have a look at TControllerInterfaceSetPropertyKind to see which properties are supported
  TControllerInterfaceSetPropertyMethod = procedure (APropertyList: PControllerInterfaceSetPropertyList); stdcall;

  // method is called by LCDHype to initialize the controller
  TControllerInterfaceInitMethod = procedure (AParameter: PControllerInterfaceInitParameter); stdcall;

  // method is called by LCDHype to free up resources used by library
  TControllerInterfaceCleanUpMethod = procedure (AParameter: PControllerInterfaceCleanUpParameter); stdcall;

  // method is called by LCDHype to send a custom char to the output device
  TControllerInterfaceSendCustomCharMethod = procedure (ACustomChar: PControllerInterfaceCustomChar); stdcall;

  // method is called by LCDHype to send a command to the output device, return true when command is being processed
  TControllerInterfaceSendCommandMethod = function (ACommand: PControllerInterfaceCommand): Boolean; stdcall;

  // method is called by LCDHype to send text the output device
  TControllerInterfaceSendTextMethod = procedure (AText: PControllerInterfaceText); stdcall;

  // method is called by LCDHype to send a bitmap the output device
  TControllerInterfaceSendBitmapMethod = procedure (ABitmap: PControllerInterfaceBitmap); stdcall;

  // method is called by LCDHype to refresh a symbol displayed by the output device
  TControllerInterfaceSendSymbolMethod = procedure (ASymbol: PControllerInterfaceSymbol); stdcall;

implementation

end.
