//
// LCDHype LCD Driver DLL Source Code
//
// Requirements: LCDHype version mentioned in ControllerInterface.pas
//
//               ScriptEngineInterface.pas and LCDHype version mentioned in this file
//               (if not needed remove all script engine declarations)
//
//               DriverLINX DLPortIO Driver installed
//               (if not needed remove the imports)
//
// $Id: Main.pas 21 2013-02-06 20:22:19Z LostSoul $
//

unit Main;

interface

uses
  ScriptEngineInterface,  // for TScriptFunctionImplementation
  ControllerInterface;    // for TControllerInterface types

var
  // pointers to controller interface functions called by LCDHype
  GetProperty: TControllerInterfaceGetPropertyMethod;
  SetProperty: TControllerInterfaceSetPropertyMethod;
  Init: TControllerInterfaceInitMethod;
  CleanUp: TControllerInterfaceCleanUpMethod;
  SendCustomChar: TControllerInterfaceSendCustomCharMethod;
  SendCommand: TControllerInterfaceSendCommandMethod;
  SendText: TControllerInterfaceSendTextMethod;
  SendBitmap: TControllerInterfaceSendBitmapMethod;
  SendSymbol: TControllerInterfaceSendSymbolMethod;

  // pointers to script engine interface functions called by LCDHype
  MyScriptFunction: TScriptFunctionImplementation;

implementation

uses
  Windows;

var
  gLibraryPath: WideString;
  gLibraryName: PChar = 'LCD Driver';
  gMyScriptFunctionReturnValue: WideString;

// imported DLL Routine from DriverLinx PortIO DLL Driver
function DlPortReadPortUchar(Port: LongWord) : Byte; stdcall; external 'DLPORTIO.DLL';

// a small routine for generating very short delays
procedure Wait(ADelay: LongWord);
var
  c: LongWord;
begin
  for c:=0 to ADelay do
    DlPortReadPortUchar($220); // read = 1 microsecond delay
end;

// start script engine interface

function Library_MyScriptFunction(const AParameter: PScriptFunctionImplementationParameter): PWideChar; stdcall;
var
  c: LongWord;
  lScriptFunctionParameter: PScriptFunctionParameter;

begin
  gMyScriptFunctionReturnValue := '';

  if AParameter.StructureSize >= SizeOf(TScriptFunctionImplementationParameter) then begin

    // simply return the parameter names and values passed to this function
    if AParameter.ParameterCount > 0 then begin
      lScriptFunctionParameter := AParameter.ParameterList;
      c := 0;
      while c < AParameter.ParameterCount do begin

        if gMyScriptFunctionReturnValue <> '' then
          gMyScriptFunctionReturnValue := gMyScriptFunctionReturnValue + '%Common.CreateNewLine()';

        gMyScriptFunctionReturnValue := gMyScriptFunctionReturnValue +
                                        lScriptFunctionParameter.Name + '=' +
                                        lScriptFunctionParameter.Value;

        Inc(lScriptFunctionParameter);
        c := c + 1;
      end;
    end;

  end;

  result := PWideChar(gMyScriptFunctionReturnValue);
end;

// end script engine interface

// start controller interface method implementation

procedure Library_GetProperty(AProperty: PControllerInterfaceGetProperty); stdcall;
var
  lResolution: PControllerInterfaceResolution;
  lByteProperty: PByte;
  lCustomCharPosition: PControllerInterfaceCustomCharPosition;

begin
  if AProperty^.StructureSize >= SizeOf(TControllerInterfaceGetProperty) then begin

    case AProperty^.Kind of
      gpName:
        AProperty^.Value := PChar(gLibraryName);

      gpTextModeResolution:
        begin
          // handle gpTextModeResolution to support txt output
          lResolution := AProperty^.Value;
          lResolution^.Width := 40;
          lResolution^.Height := 30;
        end;

      gpFontPitch:
        begin
          // fontpitch of lcd
          lByteProperty := AProperty^.Value;

          lByteProperty^ := 6;
        end;

      gpCustomCharPosition:
        begin
          lCustomCharPosition := AProperty^.Value;

          lCustomCharPosition^.Memory := lCustomCharPosition^.Internal + 128
        end;
    end;

  end;
end;

procedure Library_SetProperty(APropertyList: PControllerInterfaceSetPropertyList); stdcall;
var
  lProperty: PControllerInterfaceSetProperty;
  lCount: Integer;

begin
  if APropertyList^.StructureSize >= SizeOf(TControllerInterfaceSetPropertyList) then begin

    lProperty := APropertyList^.Element;
    lCount := 0;
    while lCount < APropertyList^.Length do begin
      if lProperty^.Kind = spLibraryPath then
        gLibraryPath := PWideChar(lProperty^.Value);

      Inc(lProperty);
      lCount := lCount + 1;
    end;

  end;
end;

procedure Library_Init(AData: PControllerInterfaceInitParameter); stdcall;
begin

end;

procedure Library_CleanUp(AData: PControllerInterfaceCleanUpParameter); stdcall;
begin

end;

procedure Library_SendCustomChar(ACustomChar: PControllerInterfaceCustomChar); stdcall;
begin

end;

function Library_SendCommand(ACommand: PControllerInterfaceCommand): Boolean; stdcall;
begin
  result := false;
end;

procedure Library_SendText(AText: PControllerInterfaceText); stdcall;
begin

end;

procedure Library_SendBitmap(ABitmap: PControllerInterfaceBitmap); stdcall;
begin

end;

procedure Library_SendSymbol(ASymbol: PControllerInterfaceSymbol); stdcall;
begin

end;

// end controller interface method implementation

initialization
  GetProperty := Library_GetProperty;
  SetProperty := Library_SetProperty;
  Init := Library_Init;
  CleanUp := Library_CleanUp;
  SendCustomChar := Library_SendCustomChar;
  SendCommand := Library_SendCommand;
  SendText := Library_SendText;
  SendBitmap := Library_SendBitmap;
  SendSymbol := Library_SendSymbol;

  MyScriptFunction := Library_MyScriptFunction;

end.

